(* This module defines the abstract syntax of System F, as produced
   by the parser. *)

(* This version of the syntax is still concrete, in the sense that
   names are identifiers. A more abstract version of the syntax is
   defined by the module [Types] and [Terms]. The conversion is
   performed by [Internalize]. *)

open Identifier

(* ------------------------------------------------------------------------- *)

(* Sorts. *)

(* We have three sorts of identifiers: term variables, type variables,
   and record labels. Each sort defines an independent namespace. *)

let term_sort : sort = (0, "term")
let type_sort : sort = (1, "type")
let label_sort : sort = (2, "label")

(* ------------------------------------------------------------------------- *)

(* Quantifiers. *)

(* There are two quantifiers: universal and existential. These
   quantifiers appear in the syntax of types. *)

type quantifier =
  | QForall
  | QExists

(* ------------------------------------------------------------------------- *)

(* Types. *)

type ftype =
  | SynTyVar of identifier
      (* a *)
  | SynTyArrow of ftype * ftype
      (* T -> T *)
  | SynTyRecord of (identifier * ftype) list
      (* { L : T; ...; L : T } *)
  | SynTyQuantified of quantifier * identifier * ftype
      (* forall a . T *)
      (* exists a . T *)

(* ------------------------------------------------------------------------- *)

(* Coercions. *)

type coercion = 
  | SynCoId
      (* id *)
  | SynCoTrans of loc_coercion * loc_coercion 
      (* c1; c2 *)
  | SynCoIntroForall
      (* +forall *)
  | SynCoElimExists
      (* -forall *)
  | SynCoElimForall of bool * ftype
      (* -forall> t | -forall< t *)
  | SynCoIntroExists of bool * ftype
      (* +exists> t | +exists< t *)
  | SynCoArrow of loc_coercion * loc_coercion
      (* c1 -> c2 *)
  | SynCoQuantifier of quantifier * identifier * loc_coercion
      (* all a. c | exists a. c *)
  | SynCoDistrib 
      (* distrib *)
  | SynCoDistribLabel of identifier
      (* distrib l *)
  | SynCoRecord of (identifier * loc_coercion) list

and loc_coercion =
    Error.location * coercion

(* ------------------------------------------------------------------------- *)

(* Terms. *)

type fterm =
  | SynTeVar of identifier
      (* x *)
  | SynTeFun of identifier * ftype * ftype option * loc_fterm
      (* fun (x : T) : T = t *)
  | SynTeRecFun of identifier * identifier * ftype * ftype * loc_fterm
      (* fun f (x : T) : T = t *)
  | SynTeApp of loc_fterm * loc_fterm
      (* t t *)
  | SynTeLet of identifier * loc_fterm * loc_fterm
      (* let x = t in t *)
  | SynTeTyAbs of identifier * loc_fterm
      (* fun a -> t *)
  | SynTeTyApp of loc_fterm * ftype
      (* t [T] *)
  | SynTeRecord of (identifier * loc_fterm) list
      (* { L = t; ...; L = t } *)
  | SynTeProj of loc_fterm * identifier
      (* t.L *)
  | SynTePack of loc_fterm * identifier * ftype
      (* pack t : exists a. T *)
  | SynTeUnpack of identifier * identifier * loc_fterm * loc_fterm
      (* unpack a, x = t in t *)
  | SynTeCoerce of loc_coercion * loc_fterm

and loc_fterm =
  Error.location * fterm

