(* This module defines the types of System F, as used internally by the
   typechecker. *)

open Atom

(* ------------------------------------------------------------------------- *)

(* Quantifiers. *)

(* There are two quantifiers: universal and existential. These
   quantifiers appear in the syntax of types. *)

type quantifier = Syntax.quantifier =
  | QForall
  | QExists

(* ------------------------------------------------------------------------- *)

(* Types. *)

(* We must adopt a representation of types that supports testing types for
   alpha-equivalence as well as abstracting and instantiating bound variables.

   Here, we choose a ``locally nameless'' representation, where free variables
   are represented as atoms and bound variables are represented as de Bruijn
   indices. This choice has the somewhat unpleasant consequence that there are
   two constructors for type variables, [TyBoundVar] and [TyFreeVar]. However,
   as far as the clients of this module are concerned, the former is non-existent:
   clients must never use it to construct a type; in return, clients can safely
   assume that it never occurs in a type.

   We distinguish between types and type contexts. A type context is a type with
   a hole. A type can be turned into a type context by replacing all occurrences
   of a certain type variable with holes. A type context is turned back into a
   type by filling all holes with a certain type. Type contexts are used in the
   representation of existential and universal types. *)

(* This representation is not particularly efficient. [fill] and [abstract]
   have linear time complexity. As a result, a simple traversal of a type has
   quadratic time complexity. This is acceptable here because we wish to
   emphasize simplicity and because our types should remain small. *)

(* For more details on the locally nameless representation of abstract syntax,
   see the paper ``Engineering Formal Metatheory'', by Brian Aydemir, Arthur
   Charguraud, Benjamin C. Pierce, Randy Pollack and Stephanie Weirich, in
   POPL 2008. *)

type ftype =
  | TyBoundVar of int
      (* this constructor must not be used by clients *)
  | TyFreeVar of atom
      (* a *)
  | TyArrow of ftype * ftype
      (* T -> T *)
  | TyRecord of ftype AtomMap.t
      (* { L : T; ...; L : T } *)
  | TyQuantified of quantifier * ftype_context
      (* forall a . T *)
      (* exists a . T *)

and ftype_context
      (* the representation of type contexts is not published, so the only way
	 to create and use them is via [abstract] and [fill] below. *)

(* ------------------------------------------------------------------------- *)

(* [equal] tells whether two types are equal up to alpha-equivalence. *)

val equal: ftype -> ftype -> bool

(* ------------------------------------------------------------------------- *)

(* [occurs a ty] tells whether the type variable [a] occurs free in the type
   [ty]. *)

val occurs: atom -> ftype -> bool

(* ------------------------------------------------------------------------- *)

(* [abstract a ty] replaces every free occurrence of the type variable
   [a] in the type [ty] with a hole, producing a type context. *)

val abstract: atom -> ftype -> ftype_context

(* [fill c ty] fills the type context [c] with the type [ty], producing a
   type. *)

val fill: ftype_context -> ftype -> ftype

(* ------------------------------------------------------------------------- *)

(* [matches c ty] tells whether the type [ty] is an instance of the
   context [c], that is, whether there exists a type [witness] such
   that [fill c witness] equals [ty]. [matching c ty] actually
   returns a witness or None ty is not matching cty. The witness is an
   option type where None means that the type_context c does not use
   its bound variable, i.e any type can be used as witness.
 *)

val matching: ftype_context -> ftype -> ftype option option
val matches: ftype_context -> ftype -> bool

(* ------------------------------------------------------------------------- *)

(* [hint c] suggests an identifier to represent the hole in the type
   context [c]. *)

val hint: ftype_context -> Identifier.identifier

(* ------------------------------------------------------------------------- *)

(* Typing environments map term variables to types. *)

type tenv

val empty: tenv
val lookup: atom -> tenv -> ftype
val bind: atom -> ftype -> tenv -> tenv

